<?php

/**
 * ABCKT Setup
 * @package ABCKT
 * @since 1.0
 */
defined('ABSPATH') || exit;

/**
 * Settings Class
 */
final class ABCKT
{

	protected static $_instance = null; 	
	public $current_time;
	public $currentDateTime = "";    
	public function __construct()
	{		
		add_action('wp_enqueue_scripts', [$this, 'abckt_scripts']);
		add_action('admin_enqueue_scripts', [$this, 'abckt_scripts']);        
		// Hook the form submission handler
		add_action('init', [$this,'abckt_save_form_data']);
		add_action('admin_print_styles', [$this,'add_custom_menu_image_css']);
		// Add custom image using CSS
		// Add this code to your theme's functions.php file or a custom plugin
		add_action('wp_ajax_btn_click_scheduale_timeframe_func_run', [$this,'btn_click_scheduale_timeframe_func_run']);
		add_action('wp_ajax_delete_campaign', [$this, 'delete_campaign']);		
		add_action('wp_ajax_nopriv_delete_campaign', [$this, 'delete_campaign']);		
		$this->currentDateTime = new DateTime();
		$this->current_time = $this->currentDateTime->format('Y-m-d H:i:s');
// 		echo $this->current_time;die();
		$this->includes();
		// Define the function to be run
		// add_action( 'init', [$this, 'scheduale_timeframe_func_run' ]);
		add_action( 'abckt_hourly_event', [$this, 'scheduale_timeframe_func_run' ]);
		// add_action( 'admin_init', [$this, 'scheduale_timeframe_func_run' ]);
		// Add custom cron schedule interval for half an hour
		add_filter( 'cron_schedules', [$this, 'abckt_half_hour_interval'] );
		
		add_filter('woocommerce_checkout_fields', [$this, 'remove_address_checkout_fields']);


		add_action("wp_ajax_abckt_activate_license", [$this, "abckt_activate_license"]);
		add_action("wp_ajax_nopriv_abckt_activate_license", [$this, "abckt_activate_license"]);
		add_action("wp_ajax_abckt_deactivate_license", [$this, "abckt_deactivate_license"]);
		add_action("wp_ajax_nopriv_abckt_deactivate_license", [$this, "abckt_deactivate_license"]);		
		add_action('admin_init', [$this, 'abckt_plugin_update_check']);
		add_action('admin_notices', [$this, 'abckt_plugin_update_notice']);
	}
    
	function redirect_cart_to_checkout() {
			if ( is_plugin_active( 'woocommerce/woocommerce.php' ) ) {
			// Check if the current page is the cart page
			if (is_cart()) {
				// Redirect to the checkout page
				wp_safe_redirect(wc_get_checkout_url());
				exit;
			}
		}
	}

    function add_custom_menu_image_css() {
    ?>
    <style>
        #adminmenu #toplevel_page_auto-blog .wp-menu-image {
            background-image: url(<?php echo esc_url( ABCKT_PLUGIN_URI . 'assets/logo/logo.png' ); ?>) !important;
            background-size: 30px 30px; /* Adjust size as needed */
            background-repeat: no-repeat;
        }
        #adminmenu #toplevel_page_auto-blog .wp-menu-image:before {
            display: none !important;
        }
    </style>
    <?php
}

	public function includes()
	{
        require_once ABCKT_PLUGIN_INC . 'class-add-admin-menu.php';
		include_once( ABSPATH . 'wp-admin/includes/plugin.php' );
	}
  
    function show_notification() {                    
            // require_once ABCKT_PLUGIN_TEMP . 'notifications.php';        
    }
    
public function scheduale_timeframe_func_run($campID = NULL) {
    global $wpdb;
    $table_name = $wpdb->prefix . 'wagp_data';		

    $wagp_data_query = "SELECT * FROM {$table_name}";

    // Check if campID is set
    if (isset($campID) && is_numeric($campID)) {
        $wagp_data_query .= $wpdb->prepare(" WHERE id = %d", $campID);
    } else {
        $DateTime = new DateTime();
		$current_time = $DateTime->format('Y-m-d H:i:s');
        $wagp_data_query .= $wpdb->prepare(" WHERE timeframe_exec <= %s", $current_time);
    }

    $results = $wpdb->get_results($wagp_data_query);

    // Check if there are any results
    if ($results) {
        foreach ($results as $result) {
            $postTitle = $postContent = "";				
            $AIResponse = $this->generate_full_article($result->prompt, $result->languages);
            
            if ($AIResponse['status'] == 'error') {
                $error = $AIResponse['message'];
            } else {
                $postTitle = $this->get_post_title_from_context($AIResponse['message']);
                $postContent = $this->get_post_content_from_context($AIResponse['message']);
                
                if (!empty($postTitle) && !empty($postContent)) {
                    $new_post = array(
                        'post_title'    => $postTitle,
                        'post_content'  => $postContent,
                        'post_status'   => 'publish', // Set to 'draft' if needed
                        'post_type'     => 'post', // Change to 'page' if needed
                        'post_category' => array($result->category) // Specify the category ID here
                    );
                    
                    // Insert the post into the database
                    $post_id = wp_insert_post($new_post);
                    $post_link = get_permalink($post_id);
                    $featured_img_url = $this->generateImage($result->prompt, $post_id);
                    $featured_img = $this->set_post_featured_img($featured_img_url, $post_id);
                }
            }

            if ($post_id) {
                $next_timeframe = $this->calculate_next_timeframe($result->timeframe);
                $data = array('timeframe_exec' => $next_timeframe);
                $where = array('id' => $result->id, 'category' => $result->category);
                $wpdb->update($table_name, $data, $where);
                $res = array('status' => 'success', 'message' => esc_html($post_link));
                echo wp_json_encode($res);
            } else {
                $res = array('status' => 'error', 'message' => esc_html($error));
                echo wp_json_encode($res);
            }
        }
    } else {
        echo esc_html("No data found!");
    }
    return;
}

// Function to generate the full article (title + content) in one API call
public function generate_full_article($keyword, $lang) {
    $prompt = "Create a unique high-end article in {$lang} on the topic '{$keyword}' including a catchy unique title then in new line, an introduction, and at least 5 sections don't write section 1 and section word before paragraph. Include headers (h2 and h3) and ensure it is completely original add hashtag before and after h2 tag and don't use : in text";

    return $this->createTextRequest($prompt);
}
public function get_post_title_from_context($context) {
    // Split the context by lines
    $lines = explode("\n", $context);

    // Return the first line as the title, stripping HTML tags
    if (isset($lines[0])) {
        return trim(strip_tags($lines[0]));
    }

    // Fallback: If no lines are present, return the entire context without HTML tags
    return trim(strip_tags($context));
}
public function get_post_content_from_context($context) {
    // Split the context by lines
    $lines = explode("\n", $context);

    // Remove the first line (title)
    array_shift($lines);

    // Initialize content
    $content = '';

    foreach ($lines as $line) {
        $line = trim($line);

        // Check if the line starts with 'H2:' or 'H3:'
        if (preg_match('/^H2:/', $line)) {
            $content .= '<h2>' . trim(substr($line, 3)) . '</h2>' . "\n";
        } elseif (preg_match('/^H3:/', $line)) {
            $content .= '<h3>' . trim(substr($line, 3)) . '</h3>' . "\n";
        } elseif (!empty($line)) {
            // Check if the line is the only line in a section
            if (count($lines) == 1) {
                $content .= '<h2>' . $line . '</h2>' . "\n";
            } else {
                // Add other lines as paragraph content
                $content .= '<p>' . $line . '</p>' . "\n";
            }
        }
    }

    // Return the formatted content
    return trim($content);
}
	public function abckt_scripts() {
    // Register and enqueue jQuery if not already registered
    // if (!wp_script_is('googleapis-jquery', 'registered')) {
    //     wp_register_script('googleapis-jquery', ABCKT_PLUGIN_URI . 'assets/js/googleapis.js', array(), '3.5.1', false);
    //     wp_enqueue_script('googleapis-jquery');
    // }

    // Generate a random version for cache busting
    $random_version = wp_generate_uuid4();

    // Enqueue plugin styles and scripts
    wp_enqueue_style('abckt-style', ABCKT_PLUGIN_URI . 'assets/css/style.css', array(), $random_version);
    wp_enqueue_script('abckt-jquery', ABCKT_PLUGIN_URI . 'assets/js/js.js', array('jquery'), $random_version, true);

    // Localize script with AJAX URL and nonces
    wp_localize_script('abckt-jquery', 'abckt_ajax', 
        array(
            'ajax_url' => admin_url('admin-ajax.php'),
            'nonces' => array(
                'btn_click_scheduale_timeframe_func_run' => wp_create_nonce('btn_click_scheduale_timeframe_func_run'),
                'delete_campaign' => wp_create_nonce('delete_campaign'),
                'abckt_activate_license' => wp_create_nonce('abckt_activate_license'),
                'abckt_deactivate_license' => wp_create_nonce('abckt_deactivate_license'),
                // Add more nonces as needed
            )
        )
    );

    // Enqueue Font Awesome and SweetAlert
    wp_enqueue_style('abckt-font-awesome', ABCKT_PLUGIN_URI . 'assets/css/font-awesome.css', array(), '5.13.0');
    wp_enqueue_script('sweetAlert', ABCKT_PLUGIN_URI . 'assets/js/sweetalert2.js', array(), '11.0.0', true);
}
	

	// Function to handle form submission and save data
	public function abckt_save_form_data() {
            if (!current_user_can('manage_options')) {
            add_filter('woocommerce_add_to_cart_redirect', [$this, 'redirect_to_checkout']);
            add_filter('woocommerce_return_to_shop_redirect', [$this, 'custom_return_to_shop_redirect']);
            add_action('template_redirect', [$this, 'redirect_cart_to_checkout']);
            }
	    if ($_SERVER['REQUEST_METHOD'] === 'POST') {
    		if (!isset( $_POST['ai_prompt'] ) && !wp_verify_nonce(sanitize_text_field(wp_unslash($_POST['ai_prompt'])), 'ai_prompt')) {
    			return false;
    		}
    		if (isset($_POST['ai_prompt'])) {
    			// Get the submitted values
    			$prompt = sanitize_text_field($_POST['prompt']);
    			$lang = sanitize_text_field($_POST['languages']);
    			$category = intval($_POST['category']);
    			$timeframe = sanitize_text_field($_POST['timeframe']);
    		
    			// Check if the prompt already exists in the database
    			global $wpdb;
    			$abckt_table_name = $wpdb->prefix . 'wagp_data';
    			$existing_prompt = $wpdb->get_var($wpdb->prepare("SELECT COUNT(*) FROM {$abckt_table_name} WHERE prompt = %s", $prompt));
    						
    			if ($existing_prompt > 0) {
    				// Prompt already exists, generate error
    				$error_message = 'The prompt already exists in the database.';
    				// Store error message in transient
    				set_transient('abckt_ai_form_error_message', $error_message, 10); // Store for 10 seconds
    			} else {
    				// Save data to the database
    				$wpdb->insert($abckt_table_name, array(
    					'prompt' => $prompt,
    					'category' => $category,
    					'languages' => $lang,
    					'timeframe' => $timeframe,
    					'timeframe_exec' => $this->calculate_next_timeframe($timeframe)
    				));
    				// Store success message in transient
    				set_transient('abckt_ai_form_success_message', 'Record Added Successfully!', 10); // Store for 10 seconds
    			}
    		
    			// Redirect back to the form page
    			wp_redirect($_SERVER['REQUEST_URI']);
    			exit;
    		}		
	    }
	}

	// Function to generate a unique article title based on the keyword
// 	public function generate_title($keyword, $lang, $title = NULL) {
// 	if($title == 'title')
// 	{
// 		$title = "generate title you never generated before please don't repeat title you previous generated make a unique High-End Article title you never generated before for {$keyword} write this in {$lang} without colon";
// 	}else if($title == 'desc')
// 	{
// 		$title = "Write Unique High-End Article you never generated before on {$keyword} at least 5 section write this in {$lang} with title also include h2 and h3 dont write section 1 etc";
// 	}
// 	else if($title == "photo")
// 	{
// 		$title = "give one synonym of {$keyword}";		
// 	}	
// 	return $title;
// 	}	

	public function calculate_next_timeframe($timeframe) {
		$currentDateTime = new DateTime();
		// Add 2 hours
		$currentDateTime = $currentDateTime->modify('+'.$timeframe.' hours');

		// Format the date and time
		return $nextDateTime = $currentDateTime->format('Y-m-d H:i:s');
	}
		

	function abckt_half_hour_interval( $schedules ) {
		// Add custom interval named 'half_hour' with a time of 30 minutes
		$schedules['half_hour'] = array(
			'interval' => 1800, // 30 minutes in seconds
			'display'  => __('Half Hour', 'autoblog-ckt')
		);

		return $schedules;
	}


// API request function remains the same
public function createTextRequest($prompt) {
    $api_params = array(
        'action' => sanitize_text_field('createTextRequest'),
        'prompt' => sanitize_text_field($prompt),
        'abckt_license_key' => sanitize_text_field(get_option('abckt_license_key')),
        'domain_name' => sanitize_text_field($_SERVER['HTTP_HOST']),
    );

    $response = wp_remote_post('https://autoblog.one/license-check/ab_post_api.php', array(
        'body' => $api_params,
        'timeout' => 3600,
        'sslverify' => false
    ));

    if (is_wp_error($response)) {
        error_log('Error: ' . $response->get_error_message());
    } else {
        $apiBody = json_decode(wp_remote_retrieve_body($response), true);
        if ($apiBody === null) {
            error_log('Error: Invalid JSON response');
        } else {
            return $apiBody;
        }
    }
    return;
}

	
	function btn_click_scheduale_timeframe_func_run() {		
            if (!isset($_POST['security']) || !wp_verify_nonce(sanitize_text_field(wp_unslash($_POST['security'])), 'btn_click_scheduale_timeframe_func_run')) {
                wp_send_json_error('Nonce verification failed.');
            }
            if (isset($_POST['campID']) && is_numeric($_POST['campID'])) {
    		    $result = $this->scheduale_timeframe_func_run(sanitize_text_field(wp_unslash($_POST['campID'])));
            }
    		wp_die(); // Always include this to terminate the script
	}

	public function generateImage($prompt, $post_id)
    {
		$api_params = array(
			'action' => sanitize_text_field( 'generateImage' ),			
			'prompt' => sanitize_text_field( $prompt ),			
			'post_id' => sanitize_text_field( $post_id ),
		);
		$response = wp_remote_post( 'https://autoblog.one/license-check/ab_post_api.php', array(
			'body' => $api_params,
			'timeout' => 3600,
			'sslverify' => false
			) );
					
		if ( is_wp_error( $response ) ) {
			echo esc_html('Error: ' . $response->get_error_message());
		} else {
			$apiBody = json_decode( wp_remote_retrieve_body( $response ), true );							
			if ( $apiBody === null ) {
				echo esc_html('Error: Invalid JSON response');
			} else {
				return $apiBody['message'];
			}
		}		
		return;

    }

public function set_post_featured_img($image_url, $post_id) {
    
	
	if (!function_exists('WP_Filesystem')) {
		require_once (ABSPATH . 'wp-admin/includes/file.php');
	}
	WP_Filesystem();

	global $wp_filesystem;

	

	// Initialize the WP_Filesystem	
    $filesystem = WP_Filesystem();
    
    // Check if the WP_Filesystem initialization was successful
    if ( ! $filesystem ) {
		// Unable to initialize WP_Filesystem
        return false;
    }
    $response = wp_remote_get($image_url);
    
    // Check for errors in the response
    if (is_wp_error($response)) {
		echo esc_html('Error retrieving image: ' . $response->get_error_message());
        return false;
    }
    
    // Retrieve the image data from the response body
    $image_data = wp_remote_retrieve_body($response);
    // Get upload directory
    $upload_dir = wp_upload_dir();
    
    // Generate unique filename
    $filename = strtotime("now") . '.jpg';
    
    // Determine file path
    $file_path = $upload_dir['path'] . '/' . $filename;
    
	// Make the remote request to fetch the image
	if (!$wp_filesystem->put_contents($file_path, $image_data, FS_CHMOD_FILE)) {
		return 'Error: Unable to save the file.';
	}
    
    // Get file type
    $wp_filetype = wp_check_filetype($filename, null);
    
    // Prepare attachment data
    $attachment = array(
		'post_mime_type' => $wp_filetype['type'],
        'post_title' => sanitize_file_name($filename),
        'post_content' => '',
        'post_status' => 'inherit'
    );
    
    // Insert attachment into WordPress media library
    $attach_id = wp_insert_attachment($attachment, $file_path, $post_id);
    
    // Include image.php for attachment metadata functions
    require_once(ABSPATH . 'wp-admin/includes/image.php');
    
    // Generate attachment metadata
    $attach_data = wp_generate_attachment_metadata($attach_id, $file_path);
    
    // Update attachment metadata
    $res1 = wp_update_attachment_metadata($attach_id, $attach_data);
    
    // Set the post thumbnail (featured image)
    $res2 = set_post_thumbnail($post_id, $attach_id);
    	
    // Check if setting the post thumbnail was successful
    if ($res1 !== false && $res2 !== false) {
		return true;
    } else {
		return false;
    }
}

	public function delete_campaign() {
		if (!isset($_POST['delete_campaign']) || !wp_verify_nonce(sanitize_text_field(wp_unslash($_POST['delete_campaign'])), 'delete_campaign')) {
			return false;
		}
		
		global $wpdb;
	
		// Retrieve campID from AJAX request
		$campID = isset($_POST['campID']) ? intval($_POST['campID']) : 0;
		
		if ($campID > 0) {
			// Table name
			$table_name = $wpdb->prefix . 'wagp_data';
			
			// Delete campaign record
			$result = $wpdb->delete($table_name, array('id' => $campID), array('%d'));
	
			if ($result !== false) {
				// Campaign deleted successfully
				echo esc_html('success');
			} else {
				// Error deleting campaign
				echo esc_html('error');
			}
		} else {
			// Invalid campID
			echo esc_html('invalid');
		}
	
		// Always include this to terminate the script
		wp_die();
	}

	public static function instance()
	{
		if (is_null(self::$_instance)) {
			self::$_instance = new self();
		}
		return self::$_instance;
	}
	
    	// Remove address fields from checkout
    function remove_address_checkout_fields($fields) {
        unset($fields['billing']['billing_company']);
        unset($fields['billing']['billing_address_1']);
        unset($fields['billing']['billing_address_2']);
        unset($fields['billing']['billing_city']);
        unset($fields['billing']['billing_postcode']);
        unset($fields['billing']['billing_country']);
        unset($fields['billing']['billing_state']);
        return $fields;
    }
    
    
    function redirect_to_checkout($url) {
        global $woocommerce;
        $checkout_url = wc_get_checkout_url();
        return $checkout_url;
    }
    
    
    function custom_return_to_shop_redirect($url) {
        return home_url(); // Redirect to the home page
    }
	
	function abckt_activate_license() {			

		if (!isset($_POST['abckt_activate_license']) || !wp_verify_nonce(sanitize_text_field(wp_unslash($_POST['abckt_activate_license'])), 'abckt_activate_license')) {
			return false;
		}
		if ( isset( $_POST['action'], $_POST['abckt_domain_name'], $_POST['abckt_license_key'] ) ) {
			
			$api_params = array(
				'action' => sanitize_text_field( $_POST['action'] ),
				'abckt_domain_name' => sanitize_text_field( $_POST['abckt_domain_name'] ),
				'abckt_license_key' => sanitize_text_field( $_POST['abckt_license_key'] ),
			);
		
			$response = wp_remote_post( 'https://autoblog.one/license-check/ab_license_validation.php', array(
				'body' => $api_params,
				'timeout' => 3600,
				'sslverify' => false
			) );
		
			if ( is_wp_error( $response ) ) {
				echo esc_html('Error: ' . $response->get_error_message());
			} else {
				$apiBody = json_decode( wp_remote_retrieve_body( $response ), true );				
				if ( $apiBody === null ) {
					echo 'Error: Invalid JSON response';
				} else {
					if($apiBody['status'] === 'success')
					{
						update_option('abckt_license_key', sanitize_text_field( $_POST['abckt_license_key'] ));
						update_option('abckt_license_status', 'active');						
					}
				}
			}
			echo wp_json_encode($apiBody);
			die();
		} else {
			echo esc_html('Error: Missing parameters');
			die();
		}
	}
	function abckt_deactivate_license() {	
        // echo wp_verify_nonce(sanitize_text_field(wp_unslash($_POST['abckt_deactivate_license'])), 'abckt_deactivate_license');die();
		if (!isset($_POST['abckt_deactivate_license']) || !wp_verify_nonce(sanitize_text_field(wp_unslash($_POST['abckt_deactivate_license'])), 'abckt_deactivate_license')) {
			return false;
		}
		if ( isset( $_POST['action'], $_POST['abckt_domain_name'], $_POST['abckt_license_key'] ) ) {
			$api_params = array(
				'action' => sanitize_text_field( $_POST['action'] ),
				'abckt_domain_name' => sanitize_text_field( $_POST['abckt_domain_name'] ),
				'abckt_license_key' => sanitize_text_field( $_POST['abckt_license_key'] ),
			);
			
			$response = wp_remote_post( 'https://autoblog.one/license-check/ab_license_validation.php', array(
				'body' => $api_params,
				'timeout' => 3600,
				'sslverify' => false
				) );
				
				if ( is_wp_error( $response ) ) {
					echo esc_html('Error: ' . $response->get_error_message());
				} else {
						
					$apiBody = json_decode( wp_remote_retrieve_body( $response ), true );				
					
				if ( $apiBody === null ) {
					echo 'Error: Invalid JSON response';
				} else {
					if($apiBody['status'] === 'success')
					{
						delete_option('abckt_license_key');
						update_option('abckt_license_status', 'Inactive');						
					}
				}
			}
			echo wp_json_encode($apiBody);
			die();
		} else {
			echo 'Error: Missing parameters';
			die();
		}
	}


	function get_membership_info(){		
		if ( !empty(get_option('abckt_license_key')) ) {
			$api_params = array(
				'action' => 'get_membership_info',		
				'abckt_domain_name' => sanitize_text_field( $_SERVER['HTTP_HOST'] ),
				'abckt_license_key' => sanitize_text_field( get_option('abckt_license_key') ),
			);
			$response = wp_remote_post( 'https://autoblog.one/license-check/ab_license_validation.php', array(
				'body' => $api_params,
				'timeout' => 3600,
				'sslverify' => false
				) );
								
				if ( is_wp_error( $response ) ) {
					echo esc_html('Error: ' . $response->get_error_message());
				} else {
						
					$apiBody = json_decode( wp_remote_retrieve_body( $response ), true );				
					
				if ( $apiBody === null ) {
					echo 'Error: Invalid JSON response';
				} else {
					if($apiBody['status'] === 'success')
					{
						return $apiBody['message'];
					}
				}
			}			
		} else {
			echo 'Error: Missing parameters';
			die();
		}

	}
	
	function check_abckt_plugin_update() {
        // URL of your update endpoint
        $update_url = 'https://example.com/update-check.php';
        
        // Get current version
        $current_version = '1.0.0'; // Update this with the current version of your plugin
        
        // Fetch update info
        $response = wp_remote_get($update_url);
        
        if (is_wp_error($response)) {
            return false;
        }
        
        $data = json_decode(wp_remote_retrieve_body($response), true);
        
        if (isset($data['version']) && version_compare($data['version'], $current_version, '>')) {
            return $data;
        }
        
        return false;
    }
	
	function abckt_plugin_update_check() {
        $update_info = $this->check_abckt_plugin_update();
        
        if ($update_info) {
            // Store the update information in a transient
            set_site_transient('abckt_plugin_update', $update_info, 12 * HOUR_IN_SECONDS);
        }
    }
    
    function abckt_plugin_update_notice() {
        $update_info = get_site_transient('abckt_plugin_update');
        
        if ($update_info) {
            $current_version = '1.0.0'; // Update this with your plugin’s version
            if (version_compare($update_info['version'], $current_version, '>')) {
                echo '<div class="notice notice-info is-dismissible">
                    <p>Your plugin is out of date. <a href="' . esc_url($update_info['download_url']) . '">Download the latest version</a>.</p>
                </div>';
            }
        }
    }

}
