<?php
namespace AutoBlogCKT;

if ( ! defined( 'ABSPATH' ) ) {
    exit; // Exit if accessed directly.
}

class Updater {

    public $plugin_version;
    public $plugin_name;
    public $plugin_slug;

    private $response_transient_key;

    public function __construct() {
        $this->plugin_version = ABCKT_PLUGIN_VERSION; // Replace with your plugin's current version
        $this->plugin_name = 'auto-blog-ckt/auto-blog-ckt.php'; // Replace with your plugin's main file
        $this->plugin_slug = 'auto-blog-ckt';
        $this->response_transient_key = md5( sanitize_key( $this->plugin_name ) . 'response_transient' );

        $this->setup_hooks();
    }

    private function setup_hooks() {
        add_filter( 'pre_set_site_transient_update_plugins', [ $this, 'check_update' ], 50 );
        add_action( 'delete_site_transient_update_plugins', [ $this, 'delete_transients' ] );
        add_filter( 'plugins_api', [ $this, 'plugins_api_filter' ], 10, 3 );

        add_action( 'update_option_WPLANG', [ $this, 'clean_get_version_cache' ] );
        add_action( 'upgrader_process_complete', [ $this, 'clean_get_version_cache' ] );
    }

    public function delete_transients() {
        $this->delete_transient( $this->response_transient_key );
    }

    private function check_transient_data( $_transient_data ) {
        if ( ! is_object( $_transient_data ) ) {
            $_transient_data = new \stdClass();
        }

        $version_info = $this->get_version_info();

        if ( is_wp_error( $version_info ) ) {
            return $_transient_data;
        }

        if ( version_compare( $this->plugin_version, $version_info['new_version'], '<' ) ) {
            $_transient_data->response[ $this->plugin_name ] = (object) [
                'slug' => $this->plugin_slug,
                'new_version' => $version_info['new_version'],
                'url' => 'https://autoblog.one/',
                'package' => 'https://autoblog.one/license-check/auto-blog-ckt.zip',
            ];
            $_transient_data->checked[ $this->plugin_name ] = $version_info['new_version'];
        } else {
            $_transient_data->no_update[ $this->plugin_name ] = (object) [
                'slug' => $this->plugin_slug,
                'new_version' => $this->plugin_version,
                'url' => 'https://autoblog.one/',
                'package' => 'https://autoblog.one/license-check/auto-blog-ckt.zip',
            ];
            $_transient_data->checked[ $this->plugin_name ] = $this->plugin_version;
        }

        $_transient_data->last_checked = current_time( 'timestamp' );

        return $_transient_data;
    }

    public function check_update( $_transient_data ) {
        return $this->check_transient_data( $_transient_data );
    }

    public function plugins_api_filter( $_data, $_action = '', $_args = null ) {
    if ( 'plugin_information' !== $_action || ! isset( $_args->slug ) || $_args->slug !== $this->plugin_slug ) {
        return $_data;
    }

    $cache_key = 'auto_blog_ckt_api_request_' . substr( md5( serialize( $this->plugin_slug ) ), 0, 15 );
    $api_request_transient = get_site_transient( $cache_key );

    if ( empty( $api_request_transient ) ) {
        $api_response = $this->get_version_info();

        if ( is_wp_error( $api_response ) ) {
            return $_data;
        }

        // Check if $api_response['sections'] is a string before unserializing
        $sections = $api_response['sections'];

        if ( is_string( $sections ) ) {
            $sections = unserialize( $sections );
        } elseif ( ! is_array( $sections ) ) {
            // Handle the case where $sections is neither a string nor an array
            return new \WP_Error('update_check_failed', __('Invalid sections data.', 'auto-blog-ckt'));
        }

        $api_request_transient = (object) [
            'name' => 'Auto Blog CKT',
            'slug' => $this->plugin_slug,
            'author' => '<a href="https://autoblog.one/">Auto Blog</a>',
            'homepage' => 'https://autoblog.one/',
            'version' => $api_response['new_version'],
            'last_updated' => $api_response['last_updated'],
            'download_link' => 'https://autoblog.one/license-check/auto-blog-ckt.zip',
            'banners' => [
                'high' => 'https://autoblog.one/license-check/banner.png',
                'low' => 'https://autoblog.one/license-check/banner.png',
            ],
            'sections' => $sections,
            'autoupdate' => true,
        ];

        set_site_transient( $cache_key, $api_request_transient, DAY_IN_SECONDS );
    }

    $_data = $api_request_transient;
    return $_data;
}


    protected function get_transient( $cache_key ) {
        return get_option( $cache_key );
    }

    protected function set_transient( $cache_key, $value, $expiration = 0 ) {
        update_option( $cache_key, [ 'timeout' => $expiration, 'value' => $value ], 'no' );
    }

    protected function delete_transient( $cache_key ) {
        delete_option( $cache_key );
    }

    public function clean_get_version_cache() {
        delete_option( 'auto_blog_ckt_' . $this->plugin_version );
    }

    private function get_version_info() {
        $update_url = 'https://autoblog.one/license-check/ckt_version_info.json';
        $response = wp_remote_get( $update_url );

        if ( is_wp_error( $response ) ) {
            return new \WP_Error( 'update_check_failed', __( 'Failed to fetch update information.', 'auto-blog-ckt' ) );
        }

        $data = json_decode( wp_remote_retrieve_body( $response ), true );

        if ( empty( $data ) ) {
            return new \WP_Error( 'update_check_failed', __( 'Invalid update information.', 'auto-blog-ckt' ) );
        }

        return $data;
    }
}
new Updater();
