<?php
if ( ! defined( 'ABSPATH' ) ) exit; // Exit if accessed directly 
// Check if user has permissions
if (!current_user_can('manage_options')) {
    return;
}

// Retrieve data from the custom table
global $wpdb;
$wagp_data = $wpdb->prefix . 'wagp_data';

// Attempt to retrieve results from cache
$results = wp_cache_get('wagp_data_results');

if ( false === $results ) {   
         
    // Execute the query and get the results
    $results = $wpdb->get_results($wpdb->prepare("SELECT * FROM {$wagp_data}"));

    // Set the results in cache for future use
    wp_cache_set('wagp_data_results', $results);
}

// Display the retrieved data in a table
echo '
<div class="wrap">';
echo '<h1>All Campaigns</h1>';
if ($results) {
    echo '<table class="widefat">';
    echo '<thead><tr><th>Prompt</th><th>Category</th><th>Language</th><th>Timeframe</th><th>Next Execution</th><th>Total Posts</th><th>Action</th></tr></thead>';
    echo '<tbody>';
    foreach ($results as $result) {
        if ($result->category > 0) {
            $args = array(
                'cat' => $result->category,
                'post_type' => 'post',
                'post_status' => 'publish',
                'posts_per_page' => -1,
            );
            $posts_query = new WP_Query($args);            
            $total_posts_count = $posts_query->found_posts;
        }    
        $category_id = $result->category; // Assuming $result is an object with category_id property
        // Get category object using category ID
        $category = get_category( $category_id );

        echo '<tr>';
        echo '<td>' . esc_html($result->prompt) . '</td>';
        echo '<td>' . esc_html($category->name) . " (" . esc_html($result->category) . ")" . '</td>';
        echo '<td>' . esc_html($result->languages) . '</td>';
        echo '<td>' . esc_html($result->timeframe) . '</td>';
        echo '<td>' . esc_html($result->timeframe_exec) . '</td>';
        echo '<td>' . esc_html($total_posts_count) . '</td>';
        echo '<td><button data-id="'. esc_html($result->id) .'" class="button btn-primary runCron">Run Schedual</button>&nbsp;<button style="color:red;border:solid 1px red;" data-id="'. esc_html($result->id) .'" class="button btn-danger delete_campaign">Delete</button></td>';
        echo '</tr>';
    }
    echo '</tbody>';
    echo '</table>';
} else {
    echo '<p>No data found.</p>';
}
echo '</div>';